#!/usr/bin/env dls-python2.4

"archiver raw XML and binary retrieval with filtering"

import datetime
import xmlrpclib
from numpy import *

def epicsTime(d):
    "converts datetime object to seconds since epics epoch"
    delta = d - datetime.datetime(1970, 1, 1)
    return int(delta.days * (24 * 60 * 60) + delta.seconds)

def pythonTime(s):
    "convert seconds past epics epoch to datetime"
    return datetime.datetime(1970, 1, 1) + datetime.timedelta(0, s)

def sinc(x):
    return sin(pi*x)/(pi*x)

def hann(N):
    n = linspace(0, N-1, N)
    return 1.0*N/(N-1)*0.5*(1-cos(2*pi*n/(N-1)))

def save(name, xs):
    f = file(name, "w")
    for x in xs:
        print >> f, x
    f.close()

def makefilter(rate = 10e3, cutoff = 100, N = 1024):
    "windowed sinc function lowpass filter design"
    w = hann(N)
    t = linspace(-N/2, N/2, N) / rate
    y = 2 * cutoff * sinc(2 * cutoff * t)
    return w * y

def main(url = 'http://archiver.pri.diamond.ac.uk/archive/cgi/ArchiveDataServer.cgi',
         KEY = 1,
         pv = "SR-DI-DCCT-01:SIGNAL",
         t0 = datetime.datetime.now() - datetime.timedelta(days = 1),
         t1 = datetime.datetime.now(),
         count = 10000,
         binary = False):
    
    lowpass = makefilter()
    
    if binary:
        method = 6
    else:
        method = 0
        
    proxy = xmlrpclib.ServerProxy(url)
    data = proxy.archiver.values(KEY, [pv], epicsTime(t0), 0, epicsTime(t1), 0, count, method)

    if binary:
        samples = fromstring(data[0]["binary"]["values"].data, 'd')
    else:
        samples = array([x["value"][0] for x in data[0]["values"]])
        
    filtered = convolve(samples, lowpass, mode = "same")
    save("filtered.txt", filtered)

if __name__ == "__main__":
    main()
